from graphene import List, Node, String
from graphene_django import DjangoObjectType

from . import choices, models


class GenomeNode(DjangoObjectType):

    class Meta:
        model = models.Genome
        interfaces = (Node, )


class ChromosomeNode(DjangoObjectType):

    class Meta:
        model = models.Chromosome
        interfaces = (Node, )


class CytoBandNode(DjangoObjectType):

    class Meta:
        model = models.CytoBand
        interfaces = (Node, )


class GeneNode(DjangoObjectType):

    status = String()
    ensembl_urls = List(String)
    refseq_urls = List(String)
    omim_urls = List(String)
    uniprot_urls = List(String)
    mgi_urls = List(String)
    rgd_urls = List(String)

    class Meta:
        model = models.Gene
        interfaces = (Node, )

    def resolve_status(self, info):
        return choices.HGNC_GENE_STATUS[self.status]

    def resolve_ensembl_urls(self, info):
        return self.ensembl_urls

    def resolve_refseq_urls(self, info):
        return self.refseq_urls

    def resolve_omim_urls(self, info):
        return self.omim_urls

    def resolve_uniprot_urls(self, info):
        return self.uniprot_urls

    def resolve_mgi_urls(self, info):
        return self.mgi_urls

    def resolve_rgd_urls(self, info):
        return self.rgd_urls


class GeneSynonymNode(DjangoObjectType):

    class Meta:
        model = models.GeneSynonym
        interfaces = (Node, )


class TranscriptNode(DjangoObjectType):

    strand = String()

    class Meta:
        model = models.Transcript
        interfaces = (Node, )

    def resolve_strand(self, info):
        return choices.STRAND_TYPES[self.strand]


class ExonNode(DjangoObjectType):

    class Meta:
        model = models.Exon
        interfaces = (Node, )



# -*- coding: utf-8 -*-
from genomix.fields import DisplayChoiceField
from rest_framework import serializers

from . import choices, models


class ChromosomeSerializer(serializers.ModelSerializer):
    """Serializer for Chromosomes."""

    genome = serializers.StringRelatedField()

    class Meta:
        model = models.Chromosome
        fields = (
            'id', 'genome', 'label', 'length', 'active'
        )


class GenomeSerializer(serializers.ModelSerializer):
    """Serializer for Genomes."""

    class Meta:
        model = models.Genome
        fields = (
            'id', 'label', 'description', 'description_url',
            'active', 'created', 'modified',
        )


class GeneSerializer(serializers.ModelSerializer):
    """Serializer for Genes."""

    status = DisplayChoiceField(choices=choices.HGNC_GENE_STATUS)
    chromosome = serializers.StringRelatedField()
    synonyms = serializers.StringRelatedField(many=True)

    class Meta:
        model = models.Gene
        fields = (
            'id', 'hgnc_id', 'symbol', 'name', 'status', 'chromosome', 'synonyms',
            'active', 'created', 'modified',
        )


class TranscriptSerializer(serializers.ModelSerializer):
    """Serializer for Transcripts."""

    gene = serializers.StringRelatedField()
    strand = DisplayChoiceField(choices=choices.STRAND_TYPES)

    class Meta:
        model = models.Transcript
        fields = (
            'id', 'label', 'gene', 'strand',
            'transcription_start', 'transcription_end', 'cds_start', 'cds_end',
            'active', 'created', 'modified',
        )


class ExonSerializer(serializers.ModelSerializer):
    """Serializer for Exons."""

    transcript = serializers.StringRelatedField()

    class Meta:
        model = models.Exon
        fields = (
            'id', 'number', 'transcript',
            'start', 'end', 'cds',
            'active', 'created', 'modified',
        )



# -*- coding: utf-8
RESOURCES = {
    'hg18': {
        'url': 'https://www.ncbi.nlm.nih.gov/grc/human/data?asm=NCBI36',
        'chromosomes': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg18/bigZips/hg18.chrom.sizes',
        'cytoband': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg18/database/cytoBand.txt.gz',
        'refgene': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg18/database/refGene.txt.gz',
        'kgXref': 'http://hgdownload.cse.ucsc.edu/goldenPath/hg18/database/kgXref.txt.gz',
    },
    'hg19': {
        'url': 'https://www.ncbi.nlm.nih.gov/grc/human/data?asm=GRCh37.p13',
        'chromosomes': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg19/bigZips/hg19.chrom.sizes',
        'cytoband': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg19/database/cytoBand.txt.gz',
        'refgene': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg19/database/refGene.txt.gz',
        'kgXref': 'http://hgdownload.cse.ucsc.edu/goldenPath/hg19/database/kgXref.txt.gz',
    },
    'hg38': {
        'url': 'https://www.ncbi.nlm.nih.gov/grc/human/data?asm=GRCh38.p10',
        'chromosomes': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg38/bigZips/hg38.chrom.sizes',
        'cytoband': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg38/database/cytoBand.txt.gz',
        'refgene': 'http://hgdownload.soe.ucsc.edu/goldenPath/hg38/database/refGene.txt.gz',
        'kgXref': 'http://hgdownload.cse.ucsc.edu/goldenPath/hg38/database/kgXref.txt.gz',
    }
}

HGNC_GENES = 'http://tinyurl.com/mqot6vl'



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext_lazy as _

from genomix.models import TimeStampedLabelModel
from model_utils.models import TimeStampedModel

from . import choices, managers


class Genome(TimeStampedLabelModel):
    """Genome Reference Build.

    Source: Downloaded from: https://www.ncbi.nlm.nih.gov/grc
    """
    description_url = models.URLField(blank=True)

    class Meta:
        verbose_name = _('Genome Build')
        verbose_name_plural = _('Genome Builds')


class Chromosome(TimeStampedModel):
    """Chromosome.

    Source: hg18 http://hgdownload.soe.ucsc.edu/goldenPath/hg18/bigZips/hg18.chrom.sizes
    Source: hg19 http://hgdownload.soe.ucsc.edu/goldenPath/hg19/bigZips/hg19.chrom.sizes
    Source: hg38 http://hgdownload.soe.ucsc.edu/goldenPath/hg38/bigZips/hg38.chrom.sizes
    """
    label = models.CharField(max_length=2)
    genome = models.ForeignKey(
        'genome.Genome',
        related_name='chromosomes',
        on_delete=models.CASCADE,
    )
    length = models.PositiveIntegerField()
    active = models.BooleanField(default=True)

    class Meta:
        verbose_name = _('Chromosome')
        verbose_name_plural = _('Chromosomes')
        unique_together = ('genome', 'label')

    def __str__(self):
        return self.label

    def save(self, **kwargs):
        self.label = self.label.upper()
        super(Chromosome, self).save(**kwargs)


class CytoBand(TimeStampedModel):
    """Cytogenetic Band.

    Source: hg18 http://hgdownload.soe.ucsc.edu/goldenPath/hg18/database/cytoBand.txt.gz
    Source: hg19 http://hgdownload.soe.ucsc.edu/goldenPath/hg19/database/cytoBand.txt.gz
    Source: hg38 http://hgdownload.soe.ucsc.edu/goldenPath/hg38/database/cytoBand.txt.gz
    """
    label = models.CharField(max_length=20)
    chromosome = models.ForeignKey(
        'genome.Chromosome',
        related_name='cytobands',
        on_delete=models.CASCADE,
    )
    start = models.PositiveIntegerField()
    end = models.PositiveIntegerField()
    stain = models.CharField(max_length=10)
    active = models.BooleanField(default=True)

    class Meta:
        verbose_name = _('Cytogenetic Band')
        verbose_name_plural = _('Cytogenetic Bands')

    def __str__(self):
        return self.label

    @property
    def locus(self):
        return 'chr{0}:{1}-{2}'.format(self.chromosome, self.start, self.end)


class Gene(TimeStampedModel):
    """Gene.

    Source: http://www.genenames.org/cgi-bin/download
    Custom Query: http://tinyurl.com/mqot6vl
    """
    symbol = models.CharField(max_length=255, db_index=True)
    name = models.TextField(blank=True)
    hgnc_id = models.PositiveIntegerField(null=True, blank=True)
    status = models.PositiveSmallIntegerField(choices=choices.HGNC_GENE_STATUS)
    active = models.BooleanField(default=True)
    chromosome = models.ForeignKey(
        'genome.Chromosome',
        related_name='genes',
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
    )

    previous_name = models.TextField(blank=True)
    synonyms = models.ManyToManyField(
        'genome.GeneSynonym',
        blank=True,
        related_name='genes',
    )

    locus_type = models.CharField(max_length=100, blank=True)
    locus_group = models.CharField(max_length=100, blank=True)

    ensembl = models.TextField(blank=True)
    refseq = models.TextField(blank=True)

    # Not curated by HGNC staff.  These are provided by exeternal resources
    not_curated_ensembl = models.TextField(
        verbose_name='Ensembl ID (supplied by Ensembl)',
        blank=True,
    )
    not_curated_refseq = models.TextField(
        verbose_name='RefSeq (supplied by NCBI)',
        blank=True,
    )
    not_curated_ucsc = models.TextField(
        verbose_name='UCSC ID (supplied by UCSC)',
        blank=True,
    )
    not_curated_omim = models.TextField(
        verbose_name='OMIM ID (supplied by OMIM)',
        blank=True,
    )
    not_curated_uniprot = models.TextField(
        verbose_name='UniProt ID (supplied by UniProt)',
        blank=True,
    )
    not_curated_mouse_genome_database = models.TextField(
        verbose_name='Mouse Genome Database ID (supplied by MGI)',
        blank=True,
    )
    not_curated_rat_genome_database = models.TextField(
        verbose_name='Rat Genome Database ID (supplied by RGD)',
        blank=True,
    )

    objects = managers.GeneQuerySet.as_manager()

    class Meta:
        verbose_name = _('Gene')
        verbose_name_plural = _('Genes')
        unique_together = ('chromosome', 'symbol')

    def __str__(self):
        return self.symbol

    def save(self, **kwargs):
        self.symbol = self.symbol.upper()
        super(Gene, self).save(**kwargs)

    @staticmethod
    def build_urls(url, field):
        if field:
            return [
                '{0}{1}'.format(url, value.strip())
                for value in field.strip().split(',')
            ]

    @property
    def ensembl_urls(self):
        return self.build_urls(
            'http://www.ensembl.org/Homo_sapiens/Gene/Summary?g=',
            self.not_curated_ensembl,
        )

    @property
    def refseq_urls(self):
        return self.build_urls(
            'https://www.ncbi.nlm.nih.gov/gene/?term=',
            self.not_curated_refseq,
        )

    @property
    def omim_urls(self):
        return self.build_urls(
            'https://www.omim.org/entry/',
            self.not_curated_omim,
        )

    @property
    def uniprot_urls(self):
        return self.build_urls(
            'http://www.uniprot.org/uniprot/',
            self.not_curated_uniprot,
        )

    @property
    def mgi_urls(self):
        return self.build_urls(
            'http://www.informatics.jax.org/marker/MGI:',
            self.not_curated_mouse_genome_database,
        )

    @property
    def rgd_urls(self):
        return self.build_urls(
            'http://rgd.mcw.edu/rgdweb/report/gene/main.html?id=',
            self.not_curated_rat_genome_database,
        )


class GeneSynonym(TimeStampedModel):
    """Other symbols used to refer to a Gene."""
    label = models.CharField(max_length=255, unique=True, db_index=True)
    active = models.BooleanField(default=True)

    class Meta:
        verbose_name = _('Gene Synonym')
        verbose_name_plural = _('Gene Synonyms')

    def __str__(self):
        return self.label


class Transcript(TimeStampedModel):
    """"Gene Transcript.

    Source: hg18 http://hgdownload.soe.ucsc.edu/goldenPath/hg18/database/refGene.txt.gz
    Source: hg19 http://hgdownload.soe.ucsc.edu/goldenPath/hg19/database/refGene.txt.gz
    Source: hg38 http://hgdownload.soe.ucsc.edu/goldenPath/hg38/database/refGene.txt.gz
    """
    label = models.CharField(max_length=255, unique=True, db_index=True)
    active = models.BooleanField(default=True)
    gene = models.ForeignKey(
        'genome.Gene',
        on_delete=models.CASCADE,
        related_name='transcripts',
    )
    strand = models.PositiveSmallIntegerField(choices=choices.STRAND_TYPES)

    transcription_start = models.PositiveIntegerField()
    transcription_end = models.PositiveIntegerField()

    cds_start = models.PositiveIntegerField()
    cds_end = models.PositiveIntegerField()

    objects = managers.TranscriptQuerySet.as_manager()

    class Meta:
        verbose_name = _('Transcript')
        verbose_name_plural = _('Transcripts')

    def __str__(self):
        return self.label


class Exon(TimeStampedModel):
    """Exon in RefSeq Transcript.

    Source: hg18 http://hgdownload.soe.ucsc.edu/goldenPath/hg18/database/refGene.txt.gz
    Source: hg19 http://hgdownload.soe.ucsc.edu/goldenPath/hg19/database/refGene.txt.gz
    Source: hg38 http://hgdownload.soe.ucsc.edu/goldenPath/hg38/database/refGene.txt.gz
    """
    number = models.PositiveSmallIntegerField()
    active = models.BooleanField(default=True)
    transcript = models.ForeignKey(
        'genome.Transcript',
        on_delete=models.CASCADE,
        related_name='exons',
    )
    start = models.PositiveIntegerField()
    end = models.PositiveIntegerField()
    cds = models.BooleanField(default=False)

    objects = managers.ExonQuerySet.as_manager()

    class Meta:
        verbose_name = _('Exon')
        verbose_name_plural = _('Exons')

    def __str__(self):
        return str(self.number)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-03-30 13:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('genome', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='gene',
            name='symbol',
            field=models.CharField(db_index=True, max_length=255),
        ),
        migrations.AlterUniqueTogether(
            name='gene',
            unique_together=set([('chromosome', 'symbol')]),
        ),
    ]
